package tree

import (
	"go.wit.com/log"
	"go.wit.com/widget"
)

func (me *TreeInfo) AddNode(a *widget.Action) *Node {
	if me.TryLock() {
		defer me.Unlock()
	} else {
		log.Info("TREE: mutex lock was already held before AddNode()")
	}
	return addNode(a)
}

func addNode(a *widget.Action) *Node {
	n := new(Node)
	n.WidgetType = a.WidgetType
	n.WidgetId = a.WidgetId
	n.ParentId = a.ParentId

	n.State = a.State
	n.State.Enable = a.State.Enable
	// n.Strings = make(map[string]int)
	// slices.Reverse(lines)
	// dropdown strings
	n.ddStrings = make([]string, 0)
	for _, s := range a.State.Strings {
		n.ddStrings = append(n.ddStrings, s)
	}

	if a.WidgetType == widget.Root {
		log.Log(TREE, "AddNode() Root")
		n.Parent = n
		treeRoot = n
		return n
	}

	if treeRoot.FindWidgetId(a.WidgetId) != nil {
		log.Log(TREEWARN, "AddNode() WidgetId already exists", a.WidgetId)
		log.Log(TREEWARN, "probably this is a Show() / Hide() issue")
		log.Log(TREEWARN, "TODO: figure out what to do here")
		return treeRoot.FindWidgetId(a.WidgetId)
	}

	// add this new widget on the binary tree
	p := treeRoot.FindWidgetId(a.ParentId)
	n.Parent = p
	if n.Parent == nil {
		log.Log(TREEWARN, "AddNode() ERROR n.Parent == nil n =", n.WidgetId, n.WidgetType, n.GetProgName())
		log.Log(TREEWARN, "AddNode() ERROR n.Parent == nil a =", a.WidgetId, a.WidgetType, a.State.ProgName)
		log.Log(TREEWARN, "AddNode() ERROR n.Parent == nil a.pid =", a.ParentId)
		return n
	}
	log.Log(TREE, "AddNode() Adding to parent =", p.ParentId, p.WidgetType, p.GetProgName())
	log.Log(TREE, "AddNode() Adding     child =", n.ParentId, n.WidgetType, n.GetProgName())
	p.children = append(p.children, n)
	return n
}

func (me *TreeInfo) DeleteNode(n *Node) {
	log.Log(TREEWARN, "DeleteNode() before lock n =", n.WidgetId, n.GetProgName())
	if me.TryLock() {
		defer me.Unlock()
	} else {
		log.Info("TREE: mutex lock was already held before DeleteNode()")
	}
	p := n.Parent
	log.Log(TREEWARN, "DeleteNode() parent =", p.WidgetId, p.GetProgName())
	for i, child := range p.children {
		log.Log(TREEWARN, "parent has child:", i, child.WidgetId, child.GetProgName())
		if n == child {
			log.Log(TREEWARN, "Found child ==", i, child.WidgetId, child.GetProgName())
			log.Log(TREEWARN, "Found n ==", i, n.WidgetId, n.GetProgName())
			p.children = append(p.children[:i], p.children[i+1:]...)
		}
	}
	for i, child := range p.children {
		log.Log(TREEWARN, "parent now has child:", i, child.WidgetId, child.GetProgName())
	}
}
